"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  SERVE_COOKIES: () => SERVE_COOKIES,
  nextReduxCookieMiddleware: () => nextReduxCookieMiddleware,
  wrapMakeStore: () => wrapMakeStore
});
module.exports = __toCommonJS(src_exports);
var import_fast_deep_equal = __toESM(require("fast-deep-equal"));
var import_next_redux_wrapper = require("next-redux-wrapper");

// src/config.ts
function processMiddlewareConfig(config) {
  const { subtrees, ...globalSubtreeConfig } = {
    ignoreStateFromStaticProps: true,
    path: "/",
    sameSite: true,
    ...config
  };
  return subtrees.map((current) => {
    if (typeof current === "string") {
      current = { subtree: current };
    }
    const {
      subtree,
      ignoreStateFromStaticProps,
      cookieName,
      defaultState,
      compress,
      serializationFunction,
      deserializationFunction,
      ...cookieOptions
    } = {
      ...globalSubtreeConfig,
      cookieName: current.subtree,
      ...current
    };
    return {
      subtree,
      ignoreStateFromStaticProps,
      cookieName,
      defaultState,
      compress: compress != null ? compress : !(serializationFunction != null ? serializationFunction : deserializationFunction),
      serializationFunction,
      deserializationFunction,
      cookieOptions
    };
  });
}

// src/cookies.ts
var import_lz_string = require("lz-string");
var import_nookies = require("nookies");

// src/util.ts
var isClient = () => typeof window !== "undefined";
var isSsrContext = (context) => context.req !== void 0 && context.res !== void 0;
var isAppContext = (context) => context.ctx !== void 0;

// src/cookies.ts
var StateCookies = class {
  static _encodeState(state, { compress, serializationFunction }) {
    const serializedState = (serializationFunction != null ? serializationFunction : JSON.stringify)(state);
    return (compress ? import_lz_string.compressToEncodedURIComponent : encodeURIComponent)(serializedState);
  }
  static _decodeState(state, { compress, deserializationFunction }) {
    const decodedState = (compress ? import_lz_string.decompressFromEncodedURIComponent : decodeURIComponent)(
      state
    );
    return (deserializationFunction != null ? deserializationFunction : JSON.parse)(decodedState);
  }
  _config = /* @__PURE__ */ new Map();
  _context;
  _cookies;
  constructor(context) {
    this._context = context;
  }
  setConfigurations(configurations) {
    for (const config of configurations) {
      this._config.set(config.cookieName, config);
    }
  }
  getAll() {
    if (typeof this._cookies === "undefined" || isClient()) {
      this._cookies = {};
      const allCookies = (0, import_nookies.parseCookies)(this._context, { decode: String });
      for (const [cookieName, cookieConfig] of this._config.entries()) {
        if (typeof allCookies[cookieName] !== "undefined") {
          this._cookies[cookieName] = StateCookies._decodeState(
            allCookies[cookieName],
            cookieConfig
          );
        } else if (typeof cookieConfig.defaultState !== "undefined") {
          this._cookies[cookieName] = cookieConfig.defaultState;
        }
      }
    }
    return this._cookies;
  }
  set(name, state) {
    const cookieConfig = this._config.get(name);
    const encodedState = StateCookies._encodeState(state, cookieConfig);
    (0, import_nookies.setCookie)(this._context, name, encodedState, {
      ...cookieConfig.cookieOptions,
      encode: String,
      httpOnly: false
    });
  }
  delete(name) {
    const { cookieOptions } = this._config.get(name);
    (0, import_nookies.destroyCookie)(this._context, name, {
      ...cookieOptions,
      httpOnly: false
    });
  }
};

// src/state-walker.ts
var import_immer = require("immer");
var import_get = __toESM(require("lodash/get"));
var import_set = __toESM(require("lodash/set"));
var walkState = (subtrees, walker, stateA, stateB) => (0, import_immer.produce)(stateA, (draftState) => {
  for (const subtreeConfig of subtrees) {
    const { subtree } = subtreeConfig;
    const result = walker(subtreeConfig, (0, import_get.default)(stateA, subtree), (0, import_get.default)(stateB, subtree));
    if (typeof result !== "undefined") {
      (0, import_set.default)(draftState, subtree, result);
    }
  }
});

// src/index.ts
var SERVE_COOKIES = "next-redux-cookie-wrapper/SERVE_COOKIES";
var wrapMakeStore = (makeStore) => (context) => {
  const store = makeStore(context);
  if (isClient()) {
    store.dispatch({ type: import_next_redux_wrapper.HYDRATE, payload: {} });
  } else {
    if (isAppContext(context)) {
      context = context.ctx;
    }
    if (isSsrContext(context)) {
      store.dispatch({
        type: SERVE_COOKIES,
        payload: new StateCookies(context)
      });
    }
  }
  return store;
};
var nextReduxCookieMiddleware = (config) => (store) => {
  const subtrees = processMiddlewareConfig(config);
  let cookies;
  if (isClient()) {
    cookies = new StateCookies();
    cookies.setConfigurations(subtrees);
  }
  return (next) => (action) => {
    switch (action.type) {
      case SERVE_COOKIES: {
        cookies = action.payload;
        cookies.setConfigurations(subtrees);
        const allCookies = cookies.getAll();
        store.dispatch({
          type: import_next_redux_wrapper.HYDRATE,
          payload: walkState(subtrees, (subtree) => allCookies[subtree.cookieName], {})
        });
        return;
      }
      case import_next_redux_wrapper.HYDRATE:
        if (isClient()) {
          const allCookies = cookies.getAll();
          action.payload = walkState(
            subtrees,
            ({ ignoreStateFromStaticProps, cookieName }) => {
              if (ignoreStateFromStaticProps) {
                return allCookies[cookieName];
              }
            },
            action.payload
          );
        }
      default: {
        const oldState = store.getState();
        const result = next(action);
        const newState = store.getState();
        const isServerSideHydrateAction = !isClient() && action.type === import_next_redux_wrapper.HYDRATE;
        if (cookies) {
          walkState(
            subtrees,
            ({ cookieName, defaultState }, oldSubtreeState, newSubtreeState) => {
              const originState = isServerSideHydrateAction ? cookies.getAll()[cookieName] : oldSubtreeState;
              if (!(0, import_fast_deep_equal.default)(originState, newSubtreeState)) {
                if (typeof defaultState !== "undefined" && (0, import_fast_deep_equal.default)(newSubtreeState, defaultState)) {
                  cookies.delete(cookieName);
                } else {
                  cookies.set(cookieName, newSubtreeState);
                }
              }
            },
            oldState,
            newState
          );
        }
        return result;
      }
    }
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SERVE_COOKIES,
  nextReduxCookieMiddleware,
  wrapMakeStore
});
//# sourceMappingURL=index.js.map